from idk.io import TranscribeData


def transcribe(payload: dict) -> TranscribeData:
    """Your code goes under here!

    Args:
        payload (dict): This dictionary is the one returned the 'transform' method of your _DeveloperInsight_ class.

    Returns:
        str: A transcription of the results of your insight. Think about whether the contents of the insight payload 
             would change the way you phrase the transcription. 
    """

    return None  # Needs to return a string!
